<?php
/**
 * @package SiteTree
 * @author Luigi Cavalieri
 * @license http://opensource.org/licenses/GPL-2.0 GPLv2.0 Public license
 * -------------------------------------------------------------------------- */


/**
 *
 * @since 1.4
 */
class SiteTreeAdminDelegate {
	/**
	 *
	 *
	 * @since 1.4
	 * @var object
	 */
	private $plugin;
	
	/**
	 * Database object reference
	 *
	 * @since 1.4
	 * @var object
	 */
	private $db;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var string
	 */
	private $pages;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var string
	 */
	private $currentPageId;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var string
	 */
	private $menuId;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var string
	 */
	private $scriptSuffix;
	
	/**
	 * @since 1.4
	 * @var object
	 */
	private static $instance;
	
	/**
	 * @since 1.4
	 * @return object
	 */
	public static function instance() {
		if (! self::$instance )
			self::$instance = new self();
		
		return self::$instance;
	}
	
	/**
	 * Initialises the properties and hooks into the WordPress admin system.
	 *
	 * @since 1.4
	 */
	public function __construct( $plugin, $pages ) {
		$this->db			= $plugin->db();
		$this->plugin		= $plugin;
		$this->pages		= $pages;
		$this->menuId		= reset( $pages )->id;
		$this->scriptSuffix = ( defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ) ? '' : '.min';
	}
	
	/**
	 *
	 * @since 1.4
	 */
	public function init( $page_view, $page_id ) {
		$this->pageView		 = $page_view;
		$this->currentPageId = $page_id;
	}
	
	/**
	 *
	 * @since 1.4
	 */
	public function catch_post_trash_event( $post_id ) {
		$post = get_post( $post_id );
		
		if ( !( $post && ( $post->post_type == 'post' ) || ( $post->post_type == 'page' ) ) )
			return false;
		
		if (! in_array( $post_id, $this->db->getOption( 'exclude', array(), 'html5', $post->post_type ) ) )
			$this->db->invalidateCache( 'html5' );
		
		if ( $this->db->getOption('include', false, 'xml', $post->post_type ) && 
			 !in_array( $post_id, $this->db->getOption( 'exclude', array(), 'xml', $post->post_type ) ) )
		{
				$this->db->invalidateCache( 'xml' );
		}
	}
	
	/**
	 *
	 * @since 1.4
	 */
	public function catch_authors_event() {
		if ( $this->db->getOption('include', false, 'html5', 'authors') || ($this->db->getOption('groupby', false, 'html5', 'post') == 'author') )
			$this->db->invalidateCache( 'html5' );
		
		if ( $this->db->getOption('include', false, 'xml', 'authors') )
			$this->db->invalidateCache( 'xml' );
	}
	
	/**
	 *
	 * @since 1.4
	 */
	public function catch_category_event() {
		if ( $this->db->getOption('include', false, 'html5', 'categories') || ($this->db->getOption('groupby', false, 'html5', 'post') == 'category') )
			$this->db->invalidateCache( 'html5' );
		
		if ( $this->db->getOption('include', false, 'xml', 'category') )
			$this->db->invalidateCache( 'xml' );
	}
	
	/**
	 *
	 * @since 1.4
	 */
	public function catch_post_tag_event() {
		if ( $this->db->getOption('include', false, 'html5', 'tags') )
			$this->db->invalidateCache( 'html5' );
		
		if ( $this->db->getOption('include', false, 'xml', 'post_tag') )
			$this->db->invalidateCache( 'xml' );
	}
	
	/**
	 *
	 * @since 1.4
	 */
	public function add_menu() {
		add_menu_page( '', 'SiteTree', 'manage_options', $this->menuId );
		
		foreach ( $this->pages as $page ) {
			if ( $page->id == $this->currentPageId ) {
				$hook_suffix = add_submenu_page( $this->menuId, $page->title, $page->menu_title, 'manage_options', $page->id, 
					array( $this->pageView, 'render' )
				);
				
				//add_action( 'admin_print_scripts-' . $hook_suffix, array( $this, 'enqueueScripts' ) );
				add_action( 'admin_print_styles-' . $hook_suffix, array( $this, 'enqueueStyles' ) );
			}
			else
				add_submenu_page( $this->menuId, $page->title, $page->menu_title, 'manage_options', $page->id, '__return_false' );
		}
	
	}
	
	/**
	 * Returns the plugin action links ('Settings', 'Deactivate').
	 *
	 * This method is hooked into the plugin_action_links filter hook.
	 *
	 * @since 1.4
	 *
	 * @param array $links
	 * @return array
	 */
	public function addActionLinks( $links ) {
		$links = array(
			'settings' => '<a href="' . SiteTreeUtilities::adminURL( $this->menuId ) . '">' . __( 'Settings', 'sitetree' ) . '</a>',
			'deactivate' => $links['deactivate']
		);
		
		return $links;
	}
	
	/**
	 * Returns the plugin meta links ('Visit plugin site', 'Donate', etc.).
	 *
	 * This method is hooked into the plugin_row_meta filter hook.
	 *
	 * @since 1.4
	 *
	 * @param array $links
	 * @param string $plugin_basename
	 * @return array
	 */
	public function addMetaLinks( $links, $basename ) {
		if ( $basename == $this->plugin->basename() )
			$links[] = '<a href="' . SiteTree::website( 'donate' ) . '" target="_blank">' . __( 'Donate', 'sitetree' ) . '</a>';
		
		return $links;
	}
	
	/**
	 *
	 * This method is hooded into the admin_print_scripts- action hook
	 *
	 *
	 * @since 1.4
	 */
	public function enqueueScripts() {
		wp_enqueue_script( 'sitetree', $this->plugin->url( 'resources/sitetree' . $this->scriptSuffix . '.js' ), array('jquery'), SiteTree::VERSION );
	}
	
	/**
	 *
	 *
	 * This method is hooded into the admin_print_styles- action hook
	 *
	 *
	 * @since 1.4
	 */
	public function enqueueStyles() {
		wp_enqueue_style( 'sitetree', $this->plugin->url( 'resources/sitetree' . $this->scriptSuffix . '.css' ), null, SiteTree::VERSION );
	}
}
?>