<?php
/**
 * @package SiteTree
 * @author Luigi Cavalieri
 * @license http://opensource.org/licenses/GPL-2.0 GPLv2.0 Public license
 * -------------------------------------------------------------------------- */

/**
 *
 *
 * @since 1.4
 */
final class SiteTreePageViewController implements SiteTreeDashboardDelegateProtocol, SiteTreeTabbedPageViewDelegateProtocol {
	/**
	 *
	 *
	 * @since 1.4
	 */
	private $plugin;
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private $db;
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private $view;
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private $activeTabId = '';
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private $formConfigMode = false;
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private $dashboardIsRendering = false;
		
	/**
	 * @since 1.0
	 * @var object
	 */
	private static $instance;
	
	/**
	 * @since 1.0
	 * @return object
	 */
	public static function instance( $plugin ) {
		if (! self::$instance )
			self::$instance = new self( $plugin );
		
		return self::$instance;
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private function __construct( $plugin ) {
		$this->plugin = $plugin;
		$this->db = $plugin->db();
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function setActiveTab( $tab_id ) { $this->activeTabId = $tab_id; }
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function enableConfigMode() { $this->formConfigMode = true; }
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function processAction( $action, $page_id, &$input = array(), $tab_id = null  ) {
		switch ( $action ) {
			case 'update':
				$dataController = $this->plugin->dataController();
				$data = $dataController->sanitiseData( $input, $page_id, $tab_id  );
				
				if ( $this->db->setOptions( $data, true ) ) {
					if ( $dataController->isGoogleRelatedPage( $page_id ) ) {
						$this->db->invalidateCache( 'xml' );
						
						if (! $this->db->getOption( 'ping' ) )
							$this->plugin->unschedulePing();
					}
					else
						$this->db->invalidateCache( 'html5' );
				}
				
				$this->registerSettingsError( $page_id, __( 'Settings %ssaved%s.', 'sitetree' ) );
				
				SiteTreeUtilities::adminRedirect( $page_id, array( 'tab' => $tab_id, 'settings-updated' => 'true' ) );
			
			case 'rebuild_google':
				$this->plugin->rebuildGoogleSitemap();
				SiteTreeUtilities::adminRedirect( $page_id );
			
			case 'rebuild_html5':
				$this->plugin->rebuildHtml5Sitemap();
				SiteTreeUtilities::adminRedirect( $page_id );
			
			case 'update_config':
				// If this is the first activation, load defaults in db
				if ( $this->db->getOption( 'page_for_sitemap', null ) === null ) {
					$defaults = $this->plugin->dataController()->html5Defaults();
					
					$this->db->setOptions( $defaults, true );
					$this->plugin->rebuildHtml5Sitemap();
				}
				
				$data = $this->plugin->dataController()->sanitiseData( $input, $page_id );
				
				$this->db->setOptions( $data, true );				
				SiteTreeUtilities::adminRedirect( $page_id );
			
			case 'enable':
				// If this is the first activation, load defaults in db
				if ( $this->db->getOption( 'enable_xml', null ) === null ) {
					$defaults = $this->plugin->dataController()->googleDefaults();
					
					$this->db->setOptions( $defaults, true );
					$this->plugin->rebuildGoogleSitemap();
				}
				
				$this->db->setOption( 'enable_xml', true );
				SiteTreeUtilities::adminRedirect( $page_id );
			
			case 'disable':
				$this->db->setOption( 'enable_xml', false );
				
				if ( $this->db->getOption( 'ping' ) )
					$this->plugin->unschedulePing();
				
				SiteTreeUtilities::adminRedirect( $page_id );
			
			case 'dismiss':
				$this->db->setOption( 'msg_displayed', true );
				SiteTreeUtilities::adminRedirect( $page_id );
		}
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function registerSettingsError(  $setting, $msg, $code = 'updated' ) {
		$errors = array();
		$errors[] = array(
			'setting' => $setting,
			'code'	  => 'msg',
			'message' => sprintf( $msg, '<span>', '</span>'),
			'type'	  => $code
		);
		
		set_transient( 'settings_errors', $errors, 30 );
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	private function getPingButton() {
		$msg   = "No pings, yet. Maybe it's time to publish something new!";
		$class = 'sitetree-ping-success';
		
		if ( $timestamp = $this->db->getOption( 'schedule', false, 'ping_status' ) ) {
			$class = 'sitetree-ping-waiting';
			$msg   = 'A new ping has been scheduled for <strong>' . SiteTreeUtilities::localTime( $timestamp ) . '</strong>';
		}
		elseif ( $timestamp = $this->db->getOption( 'date', false, 'ping_status' ) )
			$msg = 'The last ping was on <em>' . SiteTreeUtilities::localTime( $timestamp ) . '</em>';
		
		$button = '<span class=' . $class . ' id="sitetree-ping">( Ping On )<div id="sitetree-ping-status">' . $msg . '</div></span>';
		
		return $button;
	}
	
	// ------------ Delegate Methods -------------
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function dashboardCanDisplayMessage( $dashboard ) {
		$first_instal = (int) $this->db->getOption( 'instal_date' );
		
		if ( ( time() - $first_instal < 864000 ) || $this->db->getOption( 'msg_displayed' ) )
			return false;
			
		$dashboard->configureMessage(array(
			'dismiss_url' => SiteTreeUtilities::adminURL( $dashboard->id(), array( 'action' => 'dismiss' ) ),
			'head_line'	  => 'Hi, ' . wp_get_current_user()->display_name . '!'
		));
			
		return true;
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function dashboardCanRenderStats( $dashboard, $form_index ) {
		$render	= false;
		$this->dashboardIsRendering = true;
		
		if ( $form_index == 0 ) {
			$context = 'stats_xml';
			$render = $this->db->xmlEnabled();
		}
		else {
			$context = 'stats_html5';
			
			if (! $this->formConfigMode )
				$render = $this->db->html5Enabled();
		}
		
		if ( $render ) {
			$dashboard->setStats(array(
				'num_links'	=> (int) $this->db->getOption( 'url_count', 0, $context ),
				'queries'	=> (int) $this->db->getOption( 'num_queries', 0, $context ),
				'runtime'	=> (float) $this->db->getOption( 'runtime', 0, $context ),
				'date'		=> SiteTreeUtilities::localTime( $this->db->getOption( 'date', 0, $context ) )
			));
		}
		
		return $render;
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function dashboardWillRenderToolbarButtons( $dashboard, $form_index ) {
		$aux_buttons = '';
		$config		 = array();
		
		if ( $form_index == 0 && $this->db->xmlEnabled() ) {
			if ( $this->db->getOption( 'ping', true ) )
				$aux_buttons .= $this->getPingButton();
			
			$aux_buttons .= '<input type="submit" id="sitetree-disable-tb-btn" class="sitetree-aux-tb-btn sitetree-hidden-tb-btn" '
						  . 'name="submit" value="' . esc_attr__( 'Disable', 'sitetree' ) . '">';
			
			$config['rebuild_url'] = SiteTreeUtilities::adminURL( $dashboard->id(), array( 'action' => 'rebuild_google' ) );
			$config['view_url']	   = $this->plugin->googleSitemapPermalink();	
		}
		elseif ( ( $form_index != 0 ) && ( $page_for_sitemap = $this->db->getOption('page_for_sitemap') ) ) {
			$aux_buttons .= '<a href="';
			
			$config['rebuild_url'] = SiteTreeUtilities::adminURL( $dashboard->id(), array( 'action' => 'rebuild_html5' ) );
			$config['view_url']    = esc_url( get_permalink( $page_for_sitemap ) );
			
			if ( $this->formConfigMode ) {
				$config['submit_title'] = __( 'Save Changes', 'sitetree' );
				
				$aux_buttons .= SiteTreeUtilities::adminURL( $dashboard->id() );
				$aux_buttons .= '" class="sitetree-aux-tb-btn">' . __( 'Cancel', 'sitetree' );
			}
			else {
				$aux_buttons .= SiteTreeUtilities::adminURL( $dashboard->id(), array( 'edit' => 'config' ) );
				$aux_buttons .= '" class="sitetree-aux-tb-btn sitetree-hidden-tb-btn">' . __( 'Configure', 'sitetree' );
			}
			
			$aux_buttons .= '</a>';
		}
		
		$dashboard->configureToolbar( $config );
		
		return $aux_buttons;
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function tabbedPageViewTabUrl( $pageView, $tab_id ) {
		return  SiteTreeUtilities::adminURL( $pageView->id(), array( 'tab' => $tab_id ) );
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function tabbedPageViewCanSetTabActive( $index, $tab_id ) {
		if ( !$this->activeTabId && ( $index == 0 ) ) {
			$this->activeTabId = $tab_id;
			
			return true;
		}
		
		return ( $tab_id == $this->activeTabId );
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function pageViewWillRenderFormContent( $pageView ) {
		$action		   = 'update';
		$hidden_fields = wp_nonce_field( 'sitetree-options', '_sitetree_nonce', true, false );
		
		if ( $this->dashboardIsRendering ) {
			if ( $pageView->formIndex() == 0 )
				$action = $this->db->xmlEnabled() ? 'disable' : 'enable';
			else
				$action = 'update_config';
		}
		
		$hidden_fields .= '<input type="hidden" name="action" value="' . $action . '">'
		   				. '<input type="hidden" name="option_page" value="' . $pageView->id() . '">';
			   
		return $hidden_fields;
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function pageViewDidRenderFormContent( $pageView ) {
		if ( $this->activeTabId )
			return '<input type="hidden" name="tab" value="' . $this->activeTabId . '">';
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 */
	public function pageViewFieldValue( $field, $section, $tab ) {
		$context = $group = $section->id;
		
		if ( $group && isset( $tab->context )  )
			$context = $tab->context;
		
		$value	   = $this->db->getOption( $field->id, $field->default, $context, $group );
		$validator = new SiteTreeDataValidator( $value, $field->type, $field->default, $field->conditions );
		
		return $validator->value();
	}
}
?>