<?php
/**
 * @package SiteTree
 * @author Luigi Cavalieri
 * @license http://opensource.org/licenses/GPL-2.0 GPLv2.0 Public license
 * -------------------------------------------------------------------------- */


/**
 * Abstraction layer that manages the interaction with the database.
 *
 * @since 1.2
 */
final class SiteTreeDB {
	/**
	 * @since 1.4
	 * @var object
	 */
	private static $instance;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var string
	 */
	private $namespace =  'sitetree';
	
	/**
	 * 
	 *
	 * @since 1.4
	 * @var array
	 */
	private $cacheLifetime =  2592000; // 30 days;
	
	/**
	 * Options array.
	 *
	 * @since 1.2
	 * @var array
	 */
	private $options;
	
	/**
	 * Used to temporarily cache the post metadata retrieved from the database.
	 *
	 * @since 1.3
	 * @var array
	 */
	private $metadata;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var bool
	 */
	private $consolidate = false;
	
	/**
	 * @since 1.4
	 * @return object
	 */
	public static function load() {
		if ( self::$instance )
			return -1;
			
		self::$instance = new self();
		
		return self::$instance;
	}
	
	/**
	 * Initialises the @see $options property.
	 *
	 * @since 1.2
	 */
	private function __construct() {
		$this->options = get_option( $this->namespace, array() );
	}
	
	/**
	 * 
	 *
	 * @since 1.4
	 */
	public function __clone() { wp_die( __('Cheatin&#8217; huh?') ); }

	/**
	 * 
	 *
	 * @since 1.4
	 */
	public function __wakeup() { wp_die( __('Cheatin&#8217; huh?') ); }
	
	/**
	 * Updates the plugin options. 
	 *
	 * @since 1.4
	 * @param $options array
	 * @return bool
	 */
	public function setOptions( $options, $recoursive = false ) {
		if ( $recoursive ) {
			$success = false;
			
			foreach ( $options as $key => &$option ) {
				if ( $this->setOption( $key, $option ) )
					$success = true;
			}
			
			return $success;
		}
	
		if ( $this->options === $options )
			return false;
			
		SiteTree::debug();
		
		$this->options = $options;
		$this->consolidate = true;
		
		return true;
	}
	
	/**
	 * Returns the whole array of options.
	 *
	 * @since 1.4
	 * @return array
	 */
	public function getOptions() {
		return $this->options;
	}
	
	/**
	 * Stores a value into the multidimensional @see $options array and 
	 * updates the database.
	 *
	 * @since 1.4
	 * @param $key string
	 * @param $value mixed
	 * @param $context string
	 * @param $group string
	 * @return bool
	 */
	public function setOption( $key, $value, $context = null, $group = null ) {
		$old_value = $this->getOption( $key, false, $context, $group );
		
		if ( $value === $old_value  )
			return false;
			
		$this->consolidate = true;
		
		if ( $group )
			$this->options[$context][$group][$key] = $value;
		elseif ( $context )
			$this->options[$context][$key] = $value;
		else 
			$this->options[$key] = $value;
		
		return true;
	}
	
	/**
	 * Can retrieve an option value stored up to the third level into the 
	 * multidimensional @see $options array
	 *
	 * @since 1.4
	 *
	 * @param $key string
	 * @param $default mixed
	 * @param $context string
	 * @param $group string
	 * @return mixed
	 */
	public function getOption( $key, $default = false, $context = null, $group = null ) {
		if ( isset( $this->options[$key] ) && !$context )
			return $this->options[$key];
		
		if ( $group && isset( $this->options[$context][$group][$key] ) )
			return $this->options[$context][$group][$key];
			
		if ( $context && isset( $this->options[$context][$key] ) )
			return $this->options[$context][$key];
		
		return $default;
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $post_id int
	 * @param $key string
	 * @param $value mixed
	 * @return bool
	 */
	public function setPostMeta( $post_id, $key, $value ) {
		$meta_key = '_' . $this->namespace . '_' . $key;
	
		return update_metadata( 'post', $post_id, $meta_key, $value );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $post_id int
	 * @param $key string
	 * @param $default mixed
	 * @return mixed
	 */
	public function getPostMeta( $post_id, $key, $default = false ) {
		if ( strpos( $key, '_' ) !== 0 )
			$key = '_' . $this->namespace . '_' . $key;
		
		if (! isset( $this->metadata[$post_id] ) )
			$this->metadata[$post_id] = get_metadata( 'post', $post_id );
		
		if ( isset( $this->metadata[$post_id][$key][0] ) )
			return maybe_unserialize( $this->metadata[$post_id][$key][0] );
		
		return $default;
	}
	
	/**
	 *
	 *
	 * @since 1.4
	 *
	 * @param $post_id int
	 * @param $key string
	 * @return bool
	 */
	public function deletePostMeta( $post_id, $key ) {
		$meta_key = '_' . $this->namespace . '_' . $key;
		
		unset( $this->metadata[$post_id][$meta_key] );
		
		return delete_metadata( 'post', $post_id, $meta_key );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $data mixed
	 * @param $key string
	 * @return bool
	 */
	public function setCache( $key, $data ) {
		$this->setOption( $key, true, 'cache' );
		
		return $this->setTransient( $key , $data, $this->cacheLifetime );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $key string
	 * @return mixed
	 */
	public function getCache( $key ) {
		if (! $this->isCacheAlive( $key ) )
			return false;
			
		return $this->getTransient( $key );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 */
	public function invalidateCache( $key ) {
		return $this->setOption( $key, false, 'cache' );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 */
	public function isCacheAlive( $key ) {
		return (bool) $this->getOption( $key, false, 'cache' );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $data mixed
	 * @param $key string
	 * @return bool
	 */
	public function setTransient( $name, $value, $expiration = 30 ) {
		$transient_name = $this->namespace . '_' . $name;
	
		return set_transient( $transient_name , $value, $expiration );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $key string
	 * @return mixed
	 */
	public function getTransient( $name ) {
		return get_transient( $this->namespace . '_' . $name );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @param $key string
	 * @return mixed
	 */
	public function deleteTransient( $name ) {
		return delete_transient( $this->namespace . '_' . $name );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @return bool
	 */
	public function xmlEnabled() {
		return (bool) $this->getOption( 'enable_xml' );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @return bool
	 */
	public function html5Enabled() {
		return (bool) $this->getOption( 'page_for_sitemap' );
	}
	
	/**
	 *
	 * @since 1.4
	 *
	 * @return bool
	 */
	public function consolidate() {
		if ( $this->consolidate )
			update_option( $this->namespace, $this->options );
	}
}
?>