<?php
/**
 * @package SiteTree
 * @author Luigi Cavalieri
 * @license http://opensource.org/licenses/GPL-2.0 GPLv2.0 Public license
 * -------------------------------------------------------------------------- */


/**
 * 
 * @since 1.3
 */
abstract class SiteTreeFactory {
	/**
	 * Database object reference.
	 *
	 * @since 1.3
	 * @var object
	 */
	protected $db;
	
	/**
	 * What sitemap to make.
	 *
	 * @since 1.3
	 * @var string
	 */
	protected $sitemap_type = '';
	
	/**
	 *
	 * @since 1.3
	 * @var array
	 */
	protected $content_info;
	
	/**
	 *
	 *
	 * @since 1.4
	 * @var string
	 */
	protected $limit;
	
	/**
	 *
	 *
	 * @since 1.3
	 * @var array
	 */
	protected $info;
	
	/**
	 * @since 1.3
	 * @var string
	 */
	protected $sitemap = '';

	/**
	 * @since 1.3
	 * @var int
	 */
	protected $url_count = 0;
	
	/**
	 * @since 1.3
	 * @var float
	 */
	private $runtime = 0;
	
	/**
	 * @since 1.3
	 * @var int
	 */
	private $num_queries = 0;
	
	/**
	 * Initialises the properties of the class.
	 *
	 * @since 1.3
	 * @param $db object
	 */
	public function __construct($db) {
		$this->db = $db;
		$this->runtime = -microtime(true);
		$this->num_queries = -get_num_queries();
		$this->content_info = $this->db->getOption( $this->sitemap_type );
	}
	
	/**
	 * Returns the sitemap.
	 *
	 * @since 1.4
	 * @return string
	 */
	public function &getSitemap() {
		$this->loop_start();
		
		if ( is_array($this->content_info) ) {
			foreach ($this->content_info as $this->info) {
				if (! $this->get_info('include')) continue;
				
				$callback = $this->get_info('callback');
				
				if ( method_exists($this, $callback) && ($result = $this->{$callback}()) )
					$this->loop_inner($callback, $result);
			}
		}
		
		$this->loop_end();
		
		$this->runtime += microtime(true);
		$this->num_queries += get_num_queries();
		
		return $this->sitemap;
	}
	
	/**
	 *
	 * @since 1.3
	 */
	protected function loop_start() {}
	
	/**
	 *
	 * @since 1.3
	 */
	protected function loop_inner($callback, &$args) {}
	
	/**
	 *
	 * @since 1.3
	 */
	protected function loop_end() {}
	
	/**
	 *
	 * @since 1.3
	 * @return mixed
	 */
	protected function get_info($key, $default = null) {
		if ( isset($this->info[$key]) )
			return $this->info[$key];
			
		return $default;
	}
	
	/**
	 * Returns the number of URLs left to fill the sitemap.
	 *
	 * @since 1.4
	 * @return int
	 */	
	protected function getLimit() {
		return ( $this->limit - $this->url_count );
	}
	
	/**
	 *
	 * @since 1.4
	 * @return array
	 */
	public function getStats() {
		return array(
			'num_queries' => $this->num_queries,
			'runtime' => round($this->runtime, 3),
			'url_count' => $this->url_count,
			'date' => time()
		);
	}
}
?>