<?php
/**
 * @package SiteTree
 * @author Luigi Cavalieri
 * @license http://opensource.org/licenses/GPL-2.0 GPLv2.0 Public license
 * -------------------------------------------------------------------------- */


/**
 *
 *
 * @since 1.2
 */
class SiteTreeUpgrader {
	/**
	 * @since 1.4
	 */
	private $defaults;
	
	/**
	 * @since 1.4
	 */
	private $oldOptions;
	
	/**
	 * @since 1.4
	 */
	private $newOptions = array();

	/**
	 * Constructor method
	 *
	 * @since 1.2
	 */
	public function __construct( $options, $defaults ) {
		$this->oldOptions = $options;
		$this->defaults = $defaults;
	}
	
	/**
	 * @since 1.4
	 */
	public function &upgrade( $version ) {
		$version = (float) $version;
		
		if ( $version < 1.3 ) {
			$this->upgradeOptionsUpTo_1_2_2();
			
			delete_transient( 'sitetree' );
			delete_option( 'sitetree_content' );
		}
		elseif ( $version < 1.4 ) {
			$this->upgrade_1_3_Options();
			
			delete_transient( 'sitetree' );
			delete_transient( 'sitetree_xml' );
			
			wp_clear_scheduled_hook( 'sitetree_rebuild' );
			wp_clear_scheduled_hook( 'sitetree_rebuild', array('xml') );
		}
		else { return $this->oldOptions; }
		
		return $this->newOptions;
	}
	
	/**
	 * Upgrades options for versions 1.3.x
	 * 
	 * @since 1.4
	 */
	private function upgrade_1_3_Options() {
		// Migrate old options
		foreach ( $this->defaults as $key => $value ) {
			if (! is_array( $value ) ) {
				$this->newOptions[$key] = $this->getOldOption( $key, $value );
				
				continue;
			}
			
			foreach ( $value as $group_id => $group ) {
				unset( $this->oldOptions[$key][$group_id]['content_type'], $this->oldOptions[$key][$group_id]['callback'] );
				
				foreach ( $group as $def_key => $def_value )
					$this->newOptions[$key][$group_id][$def_key] = $this->getOldOption( $def_key, $def_value, $key, $group_id );
			}
		}
		
		$this->newOptions['enable_xml']  = $this->getOldOption( 'enable_xml' );
		$this->newOptions['instal_date'] = $this->getOldOption( 'first_install_date', time() );
		
		$this->newOptions['xml']['page']['exclude']   = $this->getOldOption( 'exclude', array(), 'xml', 'page' );
		$this->newOptions['xml']['post']['exclude']   = $this->getOldOption( 'exclude', array(), 'xml', 'post' );
		$this->newOptions['html5']['page']['exclude'] = $this->getOldOption( 'exclude', array(), 'html5', 'page' );
		$this->newOptions['html5']['post']['exclude'] = $this->getOldOption( 'exclude', array(), 'html5', 'post' );
		
		// Update max value of priority for all content types but pages and posts.
		$groups = array( 'category', 'authors', 'archives', 'post_tag' );
		
		foreach ( $groups as $group ) {
			if ( $this->newOptions['xml'][$group]['priority'] == '0.9' )
				$this->newOptions['xml'][$group]['priority'] = '0.8';
		}
			
	}
	
	/**
	 * Upgrades options for versions between 1.1 and 1.2.2
	 * 
	 * @since 1.4
	 */
	private function upgradeOptionsUpTo_1_2_2() {
		$this->newOptions				 = $this->defaults;
		$this->newOptions['instal_date'] = time();
	
		// Rename CSS ids and classes.
		$old_css = array('.sitemap-title', '.sitemap-list', '#sitetree-credits {font-size:90%; float:right;}');
		$new_css = array('.sitetree-title', '.sitetree-list', '#sitetree-credits {font-size:90%; text-align:right;}');
			
		$this->newOptions['css_code'] = str_replace( $old_css, $new_css, $this->getOldOption( 'css_code', '' ) );
		
		// Rename and migrate general options
		$dictionary = array(
			'page'			=> 'page_for_sitemap',
			'show_credits'	=> 'show_credits',
			'heading_tag'	=> 'title_tag',
			'wrapping_tag'	=> 'list_wrapper',
			'trailing_html'	=> 'trailing_html'
		);
		
		foreach ( $dictionary as $old_key => $new_key )
			$this->newOptions[$new_key] = $this->getOldOption( $old_key, $this->defaults[$new_key] );
		
		// Rename and migrate content options
		$this->oldOptions = get_option( 'sitetree_content', array() );
		
		$dictionaries = array(
			'authors' => array(
				'old_group_id'	=> 'authors',
				'show'			=> 'include',
				'title'			=> 'title',
				'posts_count'	=> 'show_count',
				'show_avatar'	=> 'show_avatar',
				'avatar_size'	=> 'avatar_size',
				'show_descr'	=> 'show_bio',
				'blacklist'		=> 'exclude'
			),
			'page' => array(
				'old_group_id'	=> 'pages',
				'show'			=> 'include',
				'title'			=> 'title',
				'show_home'		=> 'show_home',
				'list_style'	=> 'list_style',
				'depth'			=> 'depth'
			),
			'archives' => array(
				'old_group_id'	=> 'archives',
				'show'			=> 'include',
				'title'			=> 'title',
				'posts_count'	=> 'show_count'
			),
			'categories' => array(
				'old_group_id'	=> 'categories',
				'show'			=> 'include',
				'title'			=> 'title',
				'blacklist'		=> 'exclude',
				'posts_count'	=> 'show_count',
				'feed_text'		=> 'feed_text',
				'list_style'	=> 'list_style'
			),
			'tags' => array(
				'old_group_id'	=> 'tags',
				'show'			=> 'include',
				'title'			=> 'title',
				'posts_count'	=> 'show_count'
			),
			'post' => array(
				'old_group_id'	 => 'posts',
				'show'			 => 'include',
				'title'			 => 'title',
				'groupby'		 => 'groupby',
				'category_label' => 'category_label',
				'orderby'		 => 'orderby',
				'comments_count' => 'show_comments_count',
				'show_date'		 => 'show_date'
			)
		);
		
		foreach ( $dictionaries as $id => $dictionary ) {
			$old_group_id = $dictionary['old_group_id'];
			
			unset( $dictionary['old_group_id'] );
			
			foreach ( $dictionary as $old_key => $new_key )
				$this->newOptions['html5'][$id][$new_key] =  $this->getOldOption( $old_key, $this->defaults['html5'][$id][$new_key], $old_group_id );
		}
		
		// Convert the comma separated list of page ids into an array
		if ( $exclude = $this->getOldOption( 'blacklist', false, 'pages' ) )
			$this->newOptions['html5']['page']['exclude'] = wp_parse_id_list( $exclude );
	}
	
	/**
	 * @since 1.4
	 */
	private function getOldOption( $key, $default = false, $context = null, $group = null ) {
		if ( isset( $this->oldOptions[$key] ) && !$context )
			return $this->oldOptions[$key];
		
		if ( $group && isset( $this->oldOptions[$context][$group][$key] ) )
			return $this->oldOptions[$context][$group][$key];
			
		if ( $context && isset( $this->oldOptions[$context][$key] ) )
			return $this->oldOptions[$context][$key];
		
		return $default;
	}
}
?>