<?php
/**
 * @package SiteTree
 * @subpackage SiteTree Debugger
 * @author Luigi Cavalieri
 * @license http://opensource.org/licenses/GPL-2.0 GPLv2.0 Public license
 * -------------------------------------------------------------------------- */


class SiteTreeDebugger {
	/**
	 * Instance of the SiteTreeDebugBar class
	 *
	 * @var object
	 */
	private $debugBarPanel;
	
	/**
	 * @var string
	 */
	private $panel_title;
	
	/**
	 * Path used to truncate the file path stored into the property $caller_info.
	 *
	 * @see trace_caller_info()
	 * @var string
	 */
	private $path_base;
	
	/**
	 * @var array
	 */
	private $caller_info;
	
	/**
	 * Collection of string representations of arrays and objects.
	 *
	 * @var array
	 */
	private $structured_data = array();
	
	/**
	 * Constructor method
	 */
	public function __construct( $panel_title = '', $path_base = '' ) {
		if (! $path_base )
			$path_base = dirname( __FILE__ );
		
		if ( class_exists( 'Debug_Bar' ) )
			add_filter( 'debug_bar_panels', array( $this, 'load_debug_bar_panel' ) );
			
		$this->path_base = $path_base;
		$this->panel_title = $panel_title;
	}
	
	/**
	 * Destructor method
	 */
	public function __destruct() {
		if ( !( $this->debugBarPanel && $this->debugBarPanel->rendered() ) && $this->structured_data ) {
			echo '<pre>';
			
			foreach ( $this->structured_data as $data )
				echo esc_attr( $data ) . "\n\n\n";
				
			echo '</pre>';
		} 
	}
	
	/**
	 * @param array $panels
	 * @return array
	 */
	public function &load_debug_bar_panel( $panels ) {
		include_once( 'sitetree-debug-bar.class.php' );
		
		$this->debugBarPanel = new SiteTreeDebugBar( $this->panel_title );
		
		array_unshift( $panels, $this->debugBarPanel );
		
		return $panels;
	}
	
	/**
	 * @param mixed $data,... Unlimited optional number of data to debug
	 * @return bool
	 */
	public function debug() {
		$this->trace_caller_info();
		
		if (! ( $args = func_get_args() ) )
			return $this->output( $this->caller_info['name'], true );
			
		$arg_pos = 0;
		
		foreach ( $args as &$data ) {
			$is_struct_data = false;
			
			switch ( $data_type = gettype($data) ) {
				case 'boolean':
					$data_type = 'bool';
					$data =  $data ? 'true' : 'false';
					break;
					
				case 'array':
				case 'object':
					$is_struct_data = true;
					$data = var_export($data, true);
					$this->structured_data[] = &$data;
					break;
					
				case 'NULL':
					$data_type = 'null';
					$data = $this->caller_info['name'];
					break;
					
				case 'integer':
					$data_type = 'int';
					
				case 'double':
					if ( $data >= 1338415200 ) // 2012-05-31 -> SiteTree 1.0 published on WordPress.org
						$data .= '  =>  ' . gmdate('Y-m-d H:i:s', $data);
					break;
			}
			
			// Index of the argument that is being processed
			$index = ( $arg_pos++ > 0 ) ? ( ':' . $arg_pos ) : '';
				
			if (! $this->output( $data, false, $index . ' (' . $data_type . ')', $is_struct_data ) )
				return false;
		}
		
		return true;
	}
	
	/**
	 * @param string $msg
	 * @return bool
	 */
	public function log( $msg = '' ) {
		$this->trace_caller_info();
		
		return $this->output( $msg, true,  ' ' . $this->caller_info['name'] );
	}
		
	/**
	 * Populates the array $caller_info with a set of information about the function that 
	 * called one of the two debugger methods.
	 */
	private function trace_caller_info() {
		$index = 2;
		$backtrace = debug_backtrace();
		
		// The caller info array must be reset every time a debugger function is called.
		$this->caller_info = array( 'line' => '', 'name' => '' );
		
		// We check if the debugger method has been called by a function with the same name and
		// we set $index (of the array element from which to extract the function name) accordingly
		$num_traces = count($backtrace);
		for ( $i = $index; $i < $num_traces; $i++) {
			if ( $backtrace[$i]['function'] == $backtrace[1]['function'] ) {
				$index = $i + 1;
				break;
			}
		}
			
		if ( isset( $backtrace[$index]['class'] ) )
			$this->caller_info['name'] = $backtrace[$index]['class'] . $backtrace[$index]['type'];
		
		$this->caller_info['name'] .= $backtrace[$index]['function'] . '()';
		
		// We have to decrement $index by one to retrieve the line number and the file path
		$this->caller_info['line'] = $backtrace[--$index]['line'];
		$this->caller_info['file'] = $this->path_base ? str_replace( $this->path_base, '~', $backtrace[$index]['file']) : $backtrace[$index]['file'];
	}
	
	/**
	 * @param string $data
	 * @param bool $inline Controls how the data must be rendered into the DebugBar panel
	 * @param string $add_msg
	 * @param bool $is_struct_data
	 * @return bool
	 */
	private function output( $data, $inline, $add_msg = '', $is_struct_data = false  ) {
		$msg = '#' . $this->caller_info['line'] . $add_msg;
		
		if ( $this->debugBarPanel )
			$this->debugBarPanel->enqueue_log( $msg, $data, $this->caller_info['file'], $inline );
			
		return error_log( $msg . ' | ' . ( $is_struct_data ? '-' : $data ) );
	}
}
?>